"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElementBuilder = void 0;
const ansi_colors_1 = __importDefault(require("ansi-colors"));
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const logger_1 = require("./../libs/logger");
const variable_1 = require("./../libs/variable");
const events_manager_1 = require("./../managers/events-manager");
const root_scene_1 = require("./root-scene");
const IgnoreVarsBeforeExec = ['vars', 'setContextVarsString', 'setContextVarsObject', 'ignoreVarsBeforeExec', 'logger', 'parent', 'if', 'runs', 'elementBuilder', 'globalVars', 'scene'];
const EVAL_EXPS_KEY = Symbol('EvalExps');
class ElementBuilder {
    scene;
    elem;
    constructor(scene) {
        this.scene = scene;
    }
    logger(logger) {
        this.elem.logger = logger;
        return this;
    }
    tag(name) {
        this.elem.tag = name;
        return this;
    }
    parent(g) {
        this.elem.parent = g;
        return this;
    }
    element(ElementClazz, ...args) {
        this.elem = new ElementClazz(...args);
        this.elem.scene = this.scene;
        this.elem['$$extend'] = function (ids) {
            if (!ids?.length)
                return;
            if (typeof ids === 'string')
                ids = [ids];
            const props = {};
            ids.forEach(id => {
                const cached = root_scene_1.RootScene.templatesManager.getFromCached(id);
                cached && (0, lodash_merge_1.default)(props, cached);
            });
            return props;
        };
        this.elem['$$export'] = function (id, props) {
            if (!id)
                return;
            root_scene_1.RootScene.templatesManager.pushToCached(id, props);
            this.logger.trace(`->    \t%s`, id);
        };
        this.elem['$$clone'] = function (parent) {
            const newElement = (0, lodash_clonedeep_1.default)(this);
            newElement.parent = parent;
            return newElement;
        };
        this.elem.findParentWithType = function (ClazzType) {
            let parent = this.parent;
            do {
                if (parent instanceof ClazzType)
                    return parent;
            } while (parent = parent['parent']);
            return parent;
        };
        this.elem.markElementNeedReplace = function () {
            const props = Object.keys(this);
            let ignoreVarsBeforeExec = [...IgnoreVarsBeforeExec];
            if (this.ignoreVarsBeforeExec) {
                ignoreVarsBeforeExec.push(...this.ignoreVarsBeforeExec);
            }
            this[EVAL_EXPS_KEY] = props.filter(key => {
                return !ignoreVarsBeforeExec.includes(key) && (0, variable_1.isGetEvalExp)(this[key]);
            });
        };
        const init = this.elem.init || (() => { });
        this.elem.init = async function (elemProps) {
            events_manager_1.globalEvent.emit('element/init');
            let ignoreExec;
            if (elemProps && typeof elemProps === 'object' && !Array.isArray(elemProps)) {
                let { ignore, '<-': inheritKeys, '->': exposeKey, ...eProps } = elemProps;
                if (inheritKeys)
                    eProps = (0, lodash_merge_1.default)({}, this['$$extend'](inheritKeys), eProps);
                if (exposeKey)
                    this['$$export'](exposeKey, (0, lodash_clonedeep_1.default)(eProps));
                elemProps = eProps;
                if (elemProps.if !== undefined)
                    this.if = elemProps.if;
                if (elemProps.loop !== undefined)
                    this.loop = elemProps.loop;
                if (elemProps.async !== undefined)
                    this.async = elemProps.async;
                if (elemProps.ignoreError !== undefined)
                    this.ignoreError = elemProps.ignoreError;
                if (elemProps.log)
                    this.logger.setLevel(elemProps.log);
                if (ignore !== undefined)
                    ignoreExec = ignore;
            }
            await init.call(this, elemProps);
            this.markElementNeedReplace();
            return ignoreExec;
        };
        this.elem.setVarsAfterExec = async function () {
            if (this['vars']) {
                const keys = await this.scene.setVars(this.vars, this.setContextVarsString, this.setContextVarsObject || this);
                this.logger.trace(`[vars] \t%j`, keys.reduce((sum, e) => {
                    sum[e] = this.scene.globalVars[e];
                    return sum;
                }, {}));
            }
        };
        this.elem.evalPropsBeforeExec = async function () {
            const evalExps = this[EVAL_EXPS_KEY];
            await Promise.all(evalExps.map(async (key) => {
                this[key] = await this.scene.getVars(this[key], this);
            }));
        };
        this.elem['$$prepare'] = async function () {
            if (this.logger.is(logger_1.LoggerLevel.INFO) && this.parent?.title) {
                this.logger.addIndent();
            }
            if (this.if === undefined)
                return true;
            this.if = await this.scene.getVars(this.if, this);
            return !!this.if;
        };
        const exec = this.elem.exec;
        this.elem.exec = async function (parentData) {
            this.parentData = parentData;
            try {
                events_manager_1.globalEvent.emit('element/exec');
                await this.evalPropsBeforeExec();
                this.title && this.logger.info(`%s`, this.title);
                this.setContextVarsString = this.execResult = await exec.call(this, parentData);
            }
            catch (err) {
                this.error = err;
                if (!this.ignoreError)
                    throw this.error;
                this.logger.debug(ansi_colors_1.default.yellow(`⚠️ ${this.error?.message || this.error}`));
            }
            await this.setVarsAfterExec();
            return this.execResult;
        };
        const dispose = this.elem.dispose || (() => { });
        this.elem.dispose = async function () {
            events_manager_1.globalEvent.emit('element/dispose');
            await dispose.call(this);
        };
        return this;
    }
    build() {
        return this.elem;
    }
}
exports.ElementBuilder = ElementBuilder;
//# sourceMappingURL=element-builder.js.map